//
//  ViewController.swift
//  NuanceSDK
//
//  Created by isha.ramdasi on 12/04/22.
//

import UIKit
import NuanceMessagingCore
import NuanceMessagingUI
import NinaCobra

class ViewController: UIViewController {
    var themeDetails : MyProjectTheme.ThemeDetails!
    
    var engagementParams : EngageParameters {
        return getProdEngageParameters()
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        themeDetails = MyProjectTheme.getThemeDetails(MyProjectTheme.ThemeDarkGreen)()
        setProdAPI()
        // Do any additional setup after loading the view.
    }

    @IBAction func startChat(_ sender: UIButton) {
    
        let isChatInProgress = NuanMessaging.getInstance().getChatProgress()
        var vc:NuanceMessagingViewController;
        if(isChatInProgress) {
            vc = restoreMessagingViewController(webview: nil)
        } else {
            vc = launchMessagingViewController(params: engagementParams)
        }
        
        self.setMessageViewProperties(view: vc)
        self.setTitleBarProperties(vc: vc);
        self.setSendButtonProperties(view: vc)
        self.setInputFieldProp(view: vc)
        self.setFooterProperties(vc: vc)
        self.ninaConfiguration(vc: vc)
        vc.setMessagingDelegate(delegate: self)
        vc.modalPresentationStyle = .fullScreen
        self.present(vc, animated: true, completion: nil)
    }

    func setMessageViewProperties(view:NuanceMessagingViewController){
        
        // Styling Transcript View prop's...
        let messageView = MessagingViewProperties.init()
        messageView.messagingTranscriptBackgorundColor = themeDetails.lightBackgroundColor // Full screen background
        messageView.setSpeachBubbleView = true
        messageView.displayNinaLinksAsButtons = true
        messageView.showNameInBubbles = false
        messageView.showNameInTimeStamp = false;
        messageView.setConversationBackgroundImage = false
        messageView.isChatFromBottom = false
        messageView.groupMsgs = false
        messageView.showSurveyDialog = false
        
        // agent prop's
        messageView.agentMessageBackgroundColor = themeDetails.lightGrayColor
        messageView.agentMessageBorderColor = themeDetails.agentMsgBorderColor
        messageView.agentMessageCornerRadius = 20
        messageView.agentMessageBorderWidth = 2
        messageView.agentMessageTextColor = themeDetails.textGrayColor
        messageView.hugAgentMessageContent = true
        messageView.agentMessageRightMargin = 20
        messageView.agentMessageLeftMargin = 20
        messageView.agentTopPadding = 30
        messageView.agentRightPadding = 20
        messageView.agentBottomPadding = 30
        messageView.agentLeftPadding = 20
        messageView.agentMessageFontStyle = themeDetails.paragraphFont
        messageView.agentSpeechIcon = UIImage(named: "ic_speech_icon")?.pngData()
        messageView.ninaSpeechIcon = UIImage(named: "ic_speech_icon")?.pngData()
        // customer prop's
        
        messageView.customerMessageBackgroundColor = themeDetails.darkBackgroundColor
        messageView.customerMessageBorderColor =  themeDetails.customerMsgBorderColor
        messageView.customerMessageBorderWidth = 2
        messageView.customerMessageCornerRadius = 20
        messageView.customerMessageTextColor = UIColor.white
        messageView.customerMessageFontStyle = themeDetails.paragraphFont
        messageView.hugCustomerMessageContent = true
        messageView.customerMessageRightMargin = 10
        messageView.customerMessageLeftMargin = 10
        messageView.customerTopPadding = 20
        messageView.customerRightPadding = 20
        messageView.customerBottomPadding = 20
        messageView.customerLeftPadding = 20
        messageView.customerSpeechIcon = UIImage(named: "ic_speech_icon")?.pngData()

        messageView.showAvatarForRichWidget = true

        messageView.systemMessageFontStyle = themeDetails.paragraphFont
        messageView.agentTypingFontStyle = themeDetails.paragraphFont

        view.setTranscriptViewPropeties(properties:messageView)
    }
    
    func setFooterProperties(vc:NuanceMessagingViewController){
        let footerProp = FooterSeparationProperties.init();
        footerProp.isSeperationFooterRequired = true
        footerProp.backgroundColor = UIColor.red
    }
    
    func setTitleBarProperties(vc:NuanceMessagingViewController) {
        let title  = TitleBarProperties.init();
        title.messagingBarTitle = "Messaging"
        title.showLogo = true
        title.logoImageWidth = 30
        title.logoImageHeight = 30
        
        title.setTitlePosition = .left
        title.isTitleLogoTogetherRequired = true
        title.statusBarColorLight = false
        title.messagingBarBackgroundColor = themeDetails.topTitleColor // Title background
        title.messagingBarTitleColor = themeDetails.topTitleTextColor
        title.messagingTitleFontStyle = themeDetails.header2Font
        title.isMessagingTitleBold = true
        
        //email button properties...
        title.emailTextName = "Email"
        title.emailButtonWidth = 28
        title.emailButtonHeight = 28
        let img = UIImage(named: "ic_action_mail")
        let data1 = img?.pngData()
        title.emailImageData = data1
        
        // close button properties..
        title.closeTextName = "Close"
        title.closeButtonWidth = 17
        title.closeButtonHeight = 17
        let img1 = UIImage(named: "ic_action_close")
        let data2 = img1?.pngData()
        title.closeImageData = data2
       
        // chime(mute/unmute button)
        title.chimeTextName = "Chime"
        title.chimeButtonWidth = 23
        title.chimeButtonHeight = 23
        title.chimeTextOn = "On"
        title.chimeTextOff = "Off"
        
        let img2 = UIImage(named: "btnMuteOn")
        let data3 = img2?.pngData()
        title.chimeOnImageData = data3
        
        let img3 = UIImage(named: "btnMuteOff")
        let data4 = img3?.pngData()
        title.chimeOffImageData = data4
        
        let img4 = UIImage(named: "minimize")
        let data5 = img4?.pngData()
        title.minimizeButtonWidth = 28
        title.minimizeButtonHeight = 28
        title.minimizeImageData = data5
        title.minimizeTextName = "Minimize"

        vc.setTitleBarProperties(properties: title);
    }
    
    func setSendButtonProperties(view:NuanceMessagingViewController) {
        let sendButton = SendButtonProperties.init()
        
        sendButton.borderColor = UIColor.black
        sendButton.borderWidth = 2
        sendButton.cornerRadius = 6
        sendButton.buttonTitle = "Send"
        sendButton.disableBorderColor = themeDetails.sendBtnDisableBorderColor
        sendButton.borderColor = themeDetails.sendBtnBorderColor
                
        //GREEN
        sendButton.backgroundColor = themeDetails.sendBtnColor // Send button background
        sendButton.textColor = UIColor.white
        sendButton.useGradientEffect = false
        
        view.setSendButtonProperties(properties: sendButton)
    }
    
    func setInputFieldProp(view : NuanceMessagingViewController) {
        let inputFieldBackground = CustomerInputFieldProperties.init()
        inputFieldBackground.hintText = "Enter Your message here..."
        inputFieldBackground.borderColor = themeDetails.textFieldBorderColor
        inputFieldBackground.borderWidth = 2
        inputFieldBackground.cornerRadius = 6
        inputFieldBackground.placeHolderColor = themeDetails.textFieldTextColor
        
        inputFieldBackground.inputMessageBackgroundColor = themeDetails.textFieldBackgroundColor
        inputFieldBackground.containerBackgroundColor = themeDetails.lightBackgroundColor
        
        inputFieldBackground.textColor = themeDetails.textFieldTextColor
        inputFieldBackground.textFieldFont = themeDetails.paragraphFont
        
        
        view.setCustomerInputProperties(properties: inputFieldBackground)
    }
    
    func ninaConfiguration(vc : NuanceMessagingViewController){
        
        let serverConfiguration = NinaServerConfiguration.init()
        serverConfiguration.applicationKey = "companyName_appName"
        serverConfiguration.verificationCode = "5796d9e101d5355f5dbf95a3681f6ca5317fd9e96e5ffacac5e1305361a4eb1c"
        serverConfiguration.gateWayAddress = "webapi-demo.nuance.mobi"
        serverConfiguration.gateWayPath = "webapi-platform/websocket"
        serverConfiguration.gateWayPort = "443"
        serverConfiguration.gateWayScheme = "wss"
        vc.setNinaServerConfigurationProperties(properties: serverConfiguration)
        
        let ninaSetting = NinaSetting.init()
        ninaSetting.sessionValues.speechRecognitionCodec = "pcm_16_16k"
        ninaSetting.sessionValues.speechSynthesisCodec = "pcm_16_16k"
        ninaSetting.synthesisValues.voice = "Nathan"
        ninaSetting.serverTimeOut = 1.0
        vc.setNinaSettingProperties(properties: ninaSetting)
        
    }
    
    
    func setProdAPI() {
        NuanMessaging.getInstance()?.overrideAPIDomain("https://api-us1.digital.nuance.com");
        
        NuanMessaging.getInstance()?.overrideAuthDomain("https://auth-us1.digital.nuance.com");
        
        NuanMessaging.getInstance().initialize("west", andClientID: "ceapiClientId", andClientSecret: "ceapiClientSecret",andTagServerName:"bestbrands")
    }
    
    func getProdEngageParameters() -> EngageParameters{
        let engage = EngageParameters()
        engage.siteID = "10003715"
        engage.agentGroupID = "10004026"
        engage.businessUnitID = "13000508"
        engage.businessRuleID = "20000"
        engage.autoID = 18000492
        engage.isAsyncEngagement = false
        engage.asyncMode = "all"
        engage.customerName = "TestUser007"
        
        return engage;
        
    }
}

